(*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

exception Two_Exceptions of exn*exn

module type Error_Monad = sig

  include Monad.Monad

  val catch : (unit -> 'a) -> 'a t
  val register : exn -> 'a t
  val handle : 'a t -> 'a
  val to_option : 'a t -> 'a option
  val is_error : 'a t -> bool
  val merge : ('a -> 'b -> 'c) -> 'a t -> 'b t -> 'c t

end


module Id = struct

  include Monad.Id

  let catch af = af ()
  let register = raise
  let handle a = a
  let to_option a = Some a
  let is_error a = false
  let merge f = f

end


module Deferred_raise = struct

  type 'a dr = Good of 'a | Bad of exn

  include Monad.Monad_of_minimal (struct

    type 'a t = 'a dr
    let inj a = Good a
    let bind am f = match am with
    | Good a -> f a
    | Bad e -> Bad e

  end)

  let catch af = try  Good (af ())  with e -> Bad e
  let register e = Bad e
  let handle am = match am with
  | Good a -> a
  | Bad e -> raise e
  let to_option am = match am with
  | Good a -> Some a
  | Bad e -> None
  let is_error am = match am with
  | Good a -> false
  | Bad e -> true
  let merge f am bm = match am,bm with
  | Good a, Good b -> Good (f a b)
  | Good a, Bad e -> Bad e
  | Bad e, Good b -> Bad e
  | Bad ea, Bad eb -> Bad (Two_Exceptions (ea,eb))

end

