(*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

exception Previous_error
exception File_not_found
exception Hash_error

module Hash = Value_types.String
module Hash_cache =
  Debugable_types.Ref(Rule.TargetMap.Specialized(Value_types.Option(Hash)))

let init_cache () = ref Rule.TargetMap.empty

let get_hash cache target =
  match Rule.TargetMap.find_optional target !cache  with
  | None -> (
    let hash = if Sys.file_exists target
    then
      try match Procio.exec_read_string ("md5sum "^target) with
        | 0,hash -> Error_monad.Deferred_raise.inj hash
      with _ ->
        Error_monad.Deferred_raise.register Hash_error
    else (prerr_string ("<"^target^">");
      Error_monad.Deferred_raise.register File_not_found)  in
    cache := Rule.TargetMap.add
      target (Error_monad.Deferred_raise.to_option hash) !cache;
    Error_monad.Deferred_raise.handle hash)
  | Some None -> raise Previous_error
  | Some (Some hash) -> hash

let uncache target cache = cache := Rule.TargetMap.remove target !cache

