(*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

module Action = Value_types.String

module Actions = struct

  include Value_types.List(Action)

  let debug = List.fold_left
    (fun d -> fun a -> d^"## "^a^"\n")
    ""

end

let strings_of_actions a = a
let actions_of_strings s = s

module ActionsMap = Map.Make(Actions)

type execution = Actions.t -> bool

let print_actions actions =
  List.iter (fun a -> print_string (a^"\n")) actions;
  true

let exec_actions = List.fold_left
  (fun success -> fun action -> if success
    then Sys.command action = 0
    else false)
  true

let prompt_exec_actions = List.fold_left
  (fun success -> fun action -> if success
    then (print_string (action^"\n"); flush stdout; Sys.command action = 0)
    else false)
  true

let prompt_exec_read actions =
  let outputs = List.fold_left
    (fun outputs -> fun action -> match outputs  with
      | None -> None
      | Some outputs ->
        print_string (action^"\n"); flush stdout;
        let code,output = Procio.exec_read_string action  in
        if code=0  then Some (output::outputs)  else None)
    (Some [])
    actions  in
  Monad.Option.map (fun outputs -> List.fold_right (^) outputs "") outputs

