(*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

exception Not_translatable

module type Type = sig

  type t

  val debug : t -> string
    (* Returns some mildly human readable ASCII representation of the value. *)

end

module type Iso = sig

  type from
  type to_

  val forth : from -> to_
  val back : to_ -> from
    (* If back is partial (that is from is isomorphic to a subtype of to_),
       then Not_translatable can be raised. *)

end

module Unit : Type with type t = unit
module Bool : Type with type t = bool
module Char : Type with type t = char
module String : Type with type t = string
module Int : Type with type t = int
module Float : Type with type t = float

module Reduce(To : Type)(Iso : Iso with type to_ = To.t) :
  Type with type t=Iso.from

module Option(T : Type) : Type with type t=T.t option
module Ref(T : Type) : Type with type t=T.t ref

module Pair(Left : Type)(Right : Type) : Type with type t = Left.t * Right.t
module Triple(T1 : Type)(T2 : Type)(T3 : Type) :
  Type with type t = T1.t * T2.t * T3.t
module Tuple4(T1 : Type)(T2 : Type)(T3 : Type)(T4 : Type) :
  Type with type t = T1.t * T2.t * T3.t * T4.t
module Tuple5(T1 : Type)(T2 : Type)(T3 : Type)(T4 : Type)(T5 : Type) :
  Type with type t = T1.t * T2.t * T3.t * T4.t * T5.t

module Sum(Left : Type)(Right : Type) : Type
  with type t = (Left.t,Right.t) Sums.sum
module Sum3(T1:Type)(T2:Type)(T3:Type) : Type
  with type t = (T1.t,T2.t,T3.t) Sums.sum3

module List(Elem : Type) : Type with type t=Elem.t list
module Array(Elem : Type) : Type with type t=Elem.t array

