(*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

exception Escape_at_end
exception Unknown_escape_code

type escape_spec

val escape_spec :
  char ->                (* the escape character, commonly the backslash *)
  char list ->           (* characters which are escaped by themselves,
                            commonly this contains the escape character *)
  (char * char) list ->  (* pairs of real character/its escape code,
                            commonly this contains (newline,'n') *)
  char list ->           (* escape codes which are omitted during
                            unescaping, a common example is newline *)
  escape_spec

val escape_string : escape_spec -> string -> string
val unescape_string : escape_spec -> string -> string
val unescape_lexeme : escape_spec -> Lexing.lexbuf -> string

val line_escape_spec : escape_spec
  (* Simple spec for guaranteeing omission of '\n' in escaped strings *)

val shell_escape : string -> string
  (* Quotes/escapes a string for a POSIX shell. *)

