(*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

module type Monad_minimal = sig

  type 'a t

  val inj : 'a -> 'a t
  val bind : 'a t -> ('a -> 'b t) -> 'b t

end

module type Monad = sig

  include Monad_minimal

  val map : ('a -> 'b) -> 'a t -> 'b t
  val join : 'a t t -> 'a t

end



module Monad_of_minimal (M:Monad_minimal) = struct

  include M

  let map f am = bind am (fun a -> inj (f a))
  let join amm = bind amm (fun am -> am)

end

module Id = Monad_of_minimal (struct

  type 'a t = 'a

  let inj a = a
  let bind a f = f a

end)

module Option = Monad_of_minimal (struct

  type 'a t = 'a option

  let inj a = Some a
  let bind ao f = match ao with
  | None -> None
  | Some a -> f a

end)

