(*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

exception Not_flat


type
  node =
  | Rule of tree option Rule.drain_cases Rule.rule
  | Generated of Rule.TargetSet.t
and
  tree = node list

val fold_tree :
      (* A depth-first walk.
        'walk is the data which is folded across the walk.
        'skip is for data that is not handed to subtrees.
      *)
    ('walk -> Rule.Targets.t Rule.rule -> 'walk) ->
      (* f_rule_files *)
    ('walk -> Rule.halfrule -> 'skip * 'walk) ->
      (* f_rule_enter_scope *)
    ('skip -> 'walk -> Rule.halfrule -> 'walk) ->
      (* f_rule_exit_scope *)
    ('walk -> Rule.halfrule -> 'walk) ->
      (* f_rule_scope_undone *)
    ('walk -> Rule.TargetSet.t -> 'walk) ->
      (* f_generated *)
    'walk -> tree -> 'walk

val merge_generated : tree -> tree
val output_tree : out_channel -> tree -> unit


type
  flat_node =
  | FRule of unit Rule.drain_cases Rule.rule
  | FGenerated of Rule.TargetSet.t
and
  flat_scope = flat_node list

module Flat_Scope : Value_types.Type with type t = flat_scope

val assume_flat : tree -> flat_scope
  (* May raise Not_flat. *)

val flat_to_tree : flat_scope -> tree

val split : flat_scope ->
  Rule.Targets.t Rule.rule list *
  Rule.halfrule list *
  Rule.TargetSet.t
    (* Sorts the scope entries according to type: files rule, subscope rule,
       generation promise. *)

module Scope_Base : Value_types.Type
  with type t = flat_scope * flat_scope Rule.HalfruleMap.pt
    (* A collection of flat_scopes equivalent to a tree. 
       The first flat_scope is the root scope.
       As it is not generated by any rule, it is not part of the map. *)

val flatten : tree -> Scope_Base.t
val combine : Scope_Base.t -> tree
  (* If the same subscope rule occurs multiple times, only one occurence
     is filled by the respective subscope. *)

