(*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

module Target = Value_types.String

module TargetSet = Value_types.Set(Target)
module TargetMap = Value_types.Map(Target)

module Targets = TargetSet

type halfrule = { sources:Targets.t; actions:Action.Actions.t }
module Halfrule = struct

  include Value_types.Reduce
    (Value_types.Pair(Targets)(Action.Actions))
    (struct
      type from = halfrule
      type to_ = Targets.t * Action.Actions.t
      let forth hr = hr.sources,hr.actions
      let back (s,a) = {sources=s; actions=a}
    end)

  let debug hr =
    "< "^(Targets.debug hr.sources)^"\n"^(Action.Actions.debug hr.actions)

end

module HalfruleSet = Value_types.Set(Halfrule)
module HalfruleMap = Value_types.Map(Halfrule)

type 'a rule = { pre:halfrule; drain:'a }
module Rule = struct

  type 'a pt = 'a rule

  module Specialized(Drain : Value_types.Type) = struct

    include Value_types.Reduce
      (Value_types.Pair(Halfrule)(Drain))
      (struct
        type from = Drain.t rule
        type to_ = halfrule * Drain.t
        let forth r = r.pre,r.drain
        let back (h,d) = {pre=h; drain=d}
      end)

    let debug r =
      (Halfrule.debug r.pre)^"> "^(Drain.debug r.drain)^"\n"

  end

end

module RuleMap = Value_types.Map(Rule.Specialized(Targets))



type 'a drain_cases =
  | Files of Targets.t
  | Scope of 'a

module Drain_cases = struct

  type 'a pt = 'a drain_cases

  module Specialized(Scope : Value_types.Type) = Value_types.Reduce
    (Value_types.Sum(Targets)(Scope))
    (struct
      type from = Scope.t drain_cases
      type to_ = (Targets.t,Scope.t) Sums.sum
      let forth dc = match dc with
      | Files f -> Sums.L f
      | Scope s -> Sums.R s
      let back s = match s with
      | Sums.L f -> Files f
      | Sums.R s -> Scope s
    end)

end



let replace_drain rule drain = {pre=rule.pre; drain=drain}

