(*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

let oinmfile_name = "oinmfile"
let expanded_oinmfile_name = ".oinmfile.expanded"

let load_oinmfile_common name =
  let file = open_in name  in
  let lexbuf = Lexing.from_channel file  in
  try (
    let tree = Oinmfile_parser.oinmfile Oinmfile_lexer.oinmfile lexbuf  in
    close_in file;
    Oinmfile.merge_generated tree)
  with
  | Parsing.Parse_error -> (
    prerr_string ("Parse error in file "^name^" at position"^
      string_of_int lexbuf.Lexing.lex_curr_p.Lexing.pos_cnum^
      ".\n");
    close_in file;
    raise Parsing.Parse_error)

let load_oinmfile name = Oinmfile.assume_flat (load_oinmfile_common name)

let load_oinmfile_expanded () =
  try load_oinmfile_common expanded_oinmfile_name
  with
  | Parsing.Parse_error -> (
    prerr_string ("Warning: File "^expanded_oinmfile_name^" corrupted.\n"^
      "Using empty file instead.\n");
    flush stderr;
    [])
  | Sys_error s -> (
    prerr_string (
      "Warning: File "^expanded_oinmfile_name^" could not be opened.\n"^
      "Error message was \""^s^"\".\n"^
      "Using empty file instead.\n");
    flush stderr;
    [])

let save_oinmfile_expanded tree =
  let file = open_out expanded_oinmfile_name  in
  Oinmfile.output_tree file tree;
  close_out file

