(*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

(* Implements a data structure as used by the abstract marking algorithm.
   The terminology chosen here is that of propositional horn formulae. *)

module Formula
    (Variable : Value_types.Type)
    (VariableSet : Value_types.SetT with module Elem = Variable)
    (Clause : Value_types.Type) : sig

  include Value_types.Type

  val empty : t
  val is_empty : t -> bool
    (* The formula is empty if it has no clause left. *)
  val add : Clause.t -> VariableSet.t -> t -> t
    (* The variable set is the set of all preconditions in the clause. *)
  val retrieve_positive : t -> Clause.t option * t
    (* Returns a clause with no unmet preconditions, unless none exists.
       The clause is removed from the formula. *)
  val set : Variable.t -> t -> t
    (* In all clauses, which have the variable as a precondition,
       it is marked as met. *)

end

