/*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*/

%start oinmfile

%type <Oinmfile.tree> oinmfile
%type <Oinmfile.node list> scope
%type <Oinmfile.node> node
%type <Oinmfile.tree option Rule.drain_cases Rule.rule> rule
%type <Rule.Halfrule.t> halfrule
%type <Rule.Targets.t> sources sourcesline drains drainsline targetsline
%type <Action.Actions.t> actions
%type <string> actionline
%type <unit> emptylines

%token EOF Returntoken Sourcetoken Draintoken Begintoken Endtoken Undonetoken
%token Generatedtoken
%token <string> Line

%%

oinmfile: emptylines scope EOF
          { $2 }
        ;

scope:
       { [] }
     | node scope
       { $1::$2 }
     | Generatedtoken targetsline emptylines scope
       { (Oinmfile.Generated $2)::$4 }
     ;

node: rule
      { Oinmfile.Rule $1 }
    ;

rule: halfrule drains
      { {Rule.pre=$1; Rule.drain=Rule.Files $2} }
    | halfrule Begintoken emptylines scope Endtoken emptylines
      { {Rule.pre=$1; Rule.drain=Rule.Scope (Some $4)} }
    | halfrule Undonetoken emptylines
      { {Rule.pre=$1; Rule.drain=Rule.Scope None} }
    ;

halfrule: sources actions
      { {Rule.sources=$1; Rule.actions=$2} }
    ;

sources: sourcesline emptylines
         { $1 }
       | sourcesline emptylines sources
         { Rule.TargetSet.union $1 $3 }
       ;

sourcesline: Sourcetoken targetsline
             { $2 }
           | Sourcetoken Returntoken
             { Rule.TargetSet.empty }
           ;

actions:
         { [] }
       | actionline actions
         { $1::$2 }
       ;

actionline: Line emptylines
            { Escape.unescape_string Oinmfile_escape.action_spec $1 }
          ;

drains: drainsline emptylines
        { $1 }
      | drainsline emptylines drains
        { Rule.TargetSet.union $1 $3 }
      ;

drainsline: Draintoken targetsline
            { $2 }
          | Draintoken Returntoken
            { Rule.TargetSet.empty }
          ;

targetsline: Line
             { Oinmfile_lineparser.targets_line Oinmfile_linelexer.targets
                 (Lexing.from_string $1) }
           ;

emptylines:
            { () }
          | Returntoken emptylines
            { () }

%%

