(*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

let append options option = if options=""
  then option
  else options ^ " " ^ option

let parse_args () =
  let call::rest = Array.to_list Sys.argv  in
  let rec read_options options extras rest =
    let lexers,parsers = extras  in
    match rest with
    | "-ocamlc-options"::opts::rest ->  read_options
      {options with Options.ocamlc = append options.Options.ocamlc opts}
      extras  rest
    | "-ocamlopt-options"::opts::rest -> read_options
      {options with Options.ocamlopt = append options.Options.ocamlopt opts}
      extras  rest
    | "-ocamlcopt-options"::opts::rest -> read_options
      {options with
        Options.ocamlc = append options.Options.ocamlc opts;
        Options.ocamlopt = append options.Options.ocamlopt opts}
      extras  rest
    | "-ocamldep-options"::opts::rest -> read_options
      {options with Options.ocamldep = append options.Options.ocamldep opts}
      extras  rest
    | "-used-libs"::libs::rest -> read_options
      {options with Options.used_libs = append options.Options.used_libs libs}
      extras  rest
    | "-lexer"::base::rest -> read_options options
      (base::lexers,parsers)
      rest
    | "-parser"::base::rest -> read_options options
      (lexers,base::parsers)
      rest
    | _ -> options,extras,rest  in
  let options,(lexers,parsers),rest = read_options
    Options.empty
    ([],[])
    rest  in
  let head::tail = rest  in
  let job = match Mode.option_to_mode head, tail with
  | Some Mode.Prog, [progname; mlfile] -> Job.program progname mlfile
  | Some Mode.Prog_inner, [progname; mlfile] ->
    Job.program_inner progname mlfile
  | Some Mode.CMA_inner, [libname] -> Job.cma_inner libname
  | Some Mode.CMXA_inner, [libname] -> Job.cmxa_inner libname
  | Some Mode.Lex, [mllfile] -> Job.lex mllfile
  | Some Mode.Yacc, [mlyfile] -> Job.yacc mlyfile
  | None,_ -> Job.deduce head tail  in
  call,options,lexers,parsers,job

