(*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

exception Not_deducible

let prefix s n = String.sub s 0 n
let suffix s n = String.sub s (String.length s - n) n
let coprefix s n = String.sub s n (String.length s - n)
let cosuffix s n = String.sub s 0 (String.length s - n)

type filetype = CMI | CMO | CMX | CMA | CMXA | MLDEPS

let has_depsfile ft = match ft with
  | CMI | CMO -> false
  | CMX -> true


type fileinfo = {
  filetype : filetype;
  base : string;
  source : string option;
  target : string;
  }

let suffix_table = [
  (* Short suffixes first! *)
  ".cmi", CMI;
  ".cmo", CMO;
  ".cmx", CMX;
  ".cma", CMA;
  ".cmxa", CMXA;
  Depsfile.suffix, MLDEPS;
  ]

let deduce target =
  let suf,filetype = try
    List.find
      (fun (suf,filetype) -> suf = suffix target (String.length suf))
      suffix_table
  with
    _ -> raise Not_deducible  in
  let base = cosuffix target (String.length suf)  in
  let source = match filetype with
  | CMI -> Some (base^".mli")
  | CMO | CMX | MLDEPS -> Some (base^".ml")
  | CMA | CMXA -> None  in
  {filetype=filetype; base=base; source=source; target=target;}

let filetype info = info.filetype
let base_name info = info.base
let source_name info = match info.source with Some s -> s
let target_name info = info.target

let kill_suffix str suf =
  let l = String.length suf  in
  if (if String.length str < l  then false  else suffix str l = suf)
  then cosuffix str l
  else raise Not_found

