(*
   Copyright 2007-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

type job

val deduce : string -> string list -> job
  (* The string is the target name.
     The string list is needed for filetypes CMA, CMXA, and MLDEPS.
     Otherwise it must be empty. *)

val program : string -> string -> job
val program_inner : string -> string -> job
  (* The first string is the target name,
     the second string is the source name. *)
val cma_inner : string -> job
val cmxa_inner : string -> job
  (* The string is the target name *)
val lex : string -> job
val yacc : string -> job

val reproduce : string -> Options.options -> Rule.TargetSet.t -> job ->
  Oinmfile.flat_node
  (* The string is how the program was called.
     The target set is a set of additional dependencies. *)

val mode : job -> Mode.mode
val source_name : job -> string
val target_name : job -> string
val ocamldep_name : job -> string
  (* The name that ocamldep thinks is the target name. *)
val depsfile_name : job -> string
val given_deps : job -> string list
val extra_targets : job -> string list
val compile_command : Options.options -> job -> string
val outer_to_inner_option : job -> string
val has_source : job -> bool

