(*
   Copyright 2009-2012 by Mark Weyer
   Licensed under the OINM licence version 0
*)

module StringSet = Value_types.Set(Value_types.String)

let set_of_list = List.fold_left
  (fun set -> fun dep -> StringSet.add dep set)
  StringSet.empty

let merge names =
  (* For each name in names, the corresponding .mldeps
     is read. All these are merged into a single one, also containing
     the given strings. Order is preserved. *)
  let deps = List.map (fun name -> name, Depsfile.read_deps name) names  in
  fst (List.fold_left
    (fun (merged_list, merged_set) -> fun (name,deps) ->
      let new_list = List.filter
        (fun dep -> not (StringSet.mem dep merged_set))
        (deps @ [name])  in
      let new_set = set_of_list new_list  in
      merged_list @ new_list, StringSet.union merged_set new_set)
    ([],StringSet.empty)
    deps)

let merge_deps name deps = Depsfile.write_deps name (merge deps)

let do_merge_job job = match Job.mode job with
  | Mode.By_type Fileinfo.MLDEPS ->
    merge_deps (Job.depsfile_name job) (Job.given_deps job)

